package require mq

# For definition of Analysts_Comments, read a config file
config read nhd.cfg

set boilerplate "nhd0.met"

# Get the name of the metadata record to merge
if {$argc > 0} {
	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string compare $arg "-base"] == 0} {
			incr i
			set boilerplate [lindex $argv $i]
			} \
		else {
			lappend instances $arg
			}
		}
	}

# Read each file in turn, insert its contents into the boilerplate
# metadata, and save the metadata under the instance name

foreach filename $instances {
	# Get the base metadata
	if {[metadata m -parse $boilerplate]} {
		set eaover ""

		# Add the information from the instance

		if {[catch "open $filename r" in]} {
			puts stdout "Error: could not open data file $filename"
			exit
			} \
		else {

			# Get the lines from the input file, store in an array

			set i 0
			foreach line [split [read -nonewline $in] "\n"] {
				set lines($i) $line
				incr i
				}
			close $in
			set line_count $i

			# Calculate indentation for each line

			for {set i 0} {$i < $line_count} {incr i} {
				set t [string trim $lines($i)]
				set indent($i) [string first $t $lines($i)]
				}

			# Find the root node of the metadata

			set root [$m find_first Metadata]
			if {!$root} {
				puts stdout "Error: no root Metadata element"
				exit
				}

			# Walking the array, store each line appropriately

			set here 0
			while {$here < $line_count} {
				set t [string trim $lines($here)]

				if {$indent($here) > 0} {
					puts stdout "Warning: unexpected indentation at line [expr $here + 1]"
					}

				#---------------------------------------------------------
				# Title
				#---------------------------------------------------------

				if {[string compare $t "Title:"] == 0 && $indent($here) == 0} {
					set title ""
					set k $indent($here)
					incr here
					while {$indent($here) > $k} {
						if {[string length $title] > 0} {
							append title " "
							}
						append title [string trim $lines($here)]
						incr here
						}

					# Find the first Identification_Information, or make one

					set q [$m find_first Identification_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Identification_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Identification_Information"
							exit
							}
						}

					# Inside that, find the first Citation, or make one

					set q [$m find_in $p Citation]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Citation]
						if {!$p} {
							puts stdout "Error: failed to insert Citation"
							exit
							}
						}

					# Inside that, find the first Citation_Information, or make one

					set q [$m find_in $p Citation_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Citation_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Citation_Information"
							exit
							}
						}

					# Inside that, find the first Title

					if {[set q [$m find_in $p Title]]} {
						if {[$m parent $q] == $p} {
							# If the new title is not contained somewhere in
							# the existing title, append it to the existing title.
							set s [$m value_of -nonewline $q]
							if {[string first $title $s] == -1} {
								append s " $title"
								$m value_set $q "$s"
								}
							} \
						else {
							set q 0
							}
						}

					# If you really didn't find a title, insert one and set as its
					# value the text we just read from the file.

					if {!$q} {
						set q [$m insert -child $p Title]
						$m value_set $q "$title"
						}

					continue
					}

				#---------------------------------------------------------
				# Bounding Coordinates
				#---------------------------------------------------------

				if {[string first "Bounding Coordinate:" $t] > 0} {

					# Get all of the bounding coordinates

					set westbc  {}
					set eastbc  {}
					set northbc {}
					set southbc {}

					while {[string first "Bounding Coordinate:" $lines($here)] > 0} {
						switch $lines($here) {
							"West Bounding Coordinate:" {
								incr here
								set westbc [string trim $lines($here)]
								}
							"East Bounding Coordinate:" {
								incr here
								set eastbc [string trim $lines($here)]
								}
							"North Bounding Coordinate:" {
								incr here
								set northbc [string trim $lines($here)]
								}
							"South Bounding Coordinate:" {
								incr here
								set southbc [string trim $lines($here)]
								}
							}
						incr here
						}

					# Find the first Identification_Information, or make one

					set q [$m find_first Identification_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Identification_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Identification_Information"
							exit
							}
						}

					# Inside that, find the first Spatial_Domain, or make one

					set q [$m find_in $p Spatial_Domain]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Spatial_Domain]
						if {!$p} {
							puts stdout "Error: failed to insert Spatial_Domain"
							exit
							}
						}

					# Inside that, find the first Bounding_Coordinates, or make one

					set q [$m find_in $p Bounding_Coordinates]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Bounding_Coordinates]
						if {!$p} {
							puts stdout "Error: failed to insert Bounding_Coordinates"
							exit
							}
						}

					# Set the values of the bounding coordinates

					set q [$m find_in $p West_Bounding_Coordinate]
					if {!$q} {
						set q [$m insert -child $p West_Bounding_Coordinate]
						}
					$m value_set $q $westbc

					set q [$m find_in $p East_Bounding_Coordinate]
					if {!$q} {
						set q [$m insert -child $p East_Bounding_Coordinate]
						}
					$m value_set $q $eastbc

					set q [$m find_in $p North_Bounding_Coordinate]
					if {!$q} {
						set q [$m insert -child $p North_Bounding_Coordinate]
						}
					$m value_set $q $northbc

					set q [$m find_in $p South_Bounding_Coordinate]
					if {!$q} {
						set q [$m insert -child $p South_Bounding_Coordinate]
						}
					$m value_set $q $southbc

					continue
					}

				#---------------------------------------------------------
				# Place Keywords
				#---------------------------------------------------------

				if {[string compare $t "Place Keyword Thesaurus:"] == 0} {
					set placekt ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k && [string first "Place Keyword:" $lines($here)] == -1} {
						if {[string length $placekt] > 0} {
							append placekt " "
							}
						append placekt [string trim $lines($here)]
						incr here
						}
					set placekey {}
					while {$here < $line_count && $indent($here) > $k} {
						if {[string first "Place Keyword:" $lines($here)] == -1} {
							lappend placekey [string trim $lines($here)]
							}
						incr here
						}

					# Find the first Identification_Information, or make one

					set q [$m find_first Identification_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Identification_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Identification_Information"
							exit
							}
						}

					# Inside that, find the first Keywords, or make one

					set q [$m find_in $p Keywords]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Keywords]
						if {!$p} {
							puts stdout "Error: failed to insert Keywords"
							exit
							}
						}

					# Inside that, find the first Place, or make one

					set done 0
					set element "Place"
					set q [$m find_in $p $element]
					if {$q} {
						# For each Place, check if the thesaurus is the same
						# as the one we got from the new file

						while {$q} {
							if {[set r [$m find_in $q Place_Keyword_Thesaurus]]} {
								set s [$m value_of -nonewline $r]

								# If the placekt we just read matches this thesaurus,
								# then check the keywords; add only those we don't
								# already have.

								if {[string compare $s $placekt] == 0} {

									# Get the existing place keywords into a list

									set existing_placekey {}
									if {[set u [$m find_in $q Place_Keyword]]} {
										while {$u} {
											set s [$m value_of -nonewline $u]
											if {[string length $s] > 0} {
												lappend existing_placekey $s
												}
											set u [$m find_next $u]
											}
										}

									# Look at each new place keyword; if not in the
									# list of existing terms, add it.

									foreach item $placekey {
										if {[lsearch -exact $existing_placekey $item] == -1} {
											if {[set v [$m insert -child $q Place_Keyword]]} {
												$m value_set $v "$item"
												}
											}
										}
									set done 1
									break
									}
								}
							set q [$m find_next $q]
							}
						}

					if {$done == 0} {
						set q [$m insert -child $p $element]
						if {!$q} {
							puts stdout "Error: failed to insert $element"
							exit
							}

						# Insert the thesaurus

						set r [$m insert -child $q Place_Keyword_Thesaurus]
						$m value_set $r "$placekt"

						# Insert each place keyword

						foreach item $placekey {
							set r [$m insert -child $q Place_Keyword]
							$m value_set $r "$item"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Data Set Credit
				#---------------------------------------------------------

				if {[string compare $t "Data Set Credit:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set datacred "$text"

					# Find the first Identification_Information, or make one

					set q [$m find_first Identification_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Identification_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Identification_Information"
							exit
							}
						}

					# Inside that, find the first Data_Set_Credit, or make one

					set q [$m find_in $p Data_Set_Credit]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Data_Set_Credit]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Set_Credit"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$datacred"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $datacred $s] != 0} {
							append s "  $datacred"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Attribute Accuracy Report:
				#---------------------------------------------------------

				if {[string compare $t "Attribute Accuracy Report:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set attraccr "$text"

					# Find the first Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the first Attribute_Accuracy, or make one

					set q [$m find_in $p Attribute_Accuracy]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Attribute_Accuracy]
						if {!$p} {
							puts stdout "Error: failed to insert Attribute_Accuracy"
							exit
							}
						}

					# Inside that, find the first Attribute_Accuracy_Report, or make one

					set q [$m find_in $p Attribute_Accuracy_Report]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Attribute_Accuracy_Report]
						if {!$p} {
							puts stdout "Error: failed to insert Attribute_Accuracy_Report"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$attraccr"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $attraccr $s] != -1} {
							append s "  $attraccr"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Logical Consistency Report:
				#---------------------------------------------------------

				if {[string compare $t "Logical Consistency Report:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set logic "$text"

					# Find the first Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the first Logical_Consistency_Report, or make one

					set q [$m find_in $p Logical_Consistency_Report]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Logical_Consistency_Report]
						if {!$p} {
							puts stdout "Error: failed to insert Logical_Consistency_Report"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$logic"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $logic $s] != -1} {
							append s "  $logic"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Completeness Report:
				#---------------------------------------------------------

				if {[string compare $t "Completeness Report:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set complete "$text"

					# Find the first Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the first Completeness_Report, or make one

					set q [$m find_in $p Completeness_Report]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Completeness_Report]
						if {!$p} {
							puts stdout "Error: failed to insert Completeness_Report"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$complete"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $complete $s] != -1} {
							append s "  $complete"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Horizontal Positional Accuracy Report:
				#---------------------------------------------------------

				if {[string compare $t "Horizontal Positional Accuracy Report:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set horizpar "$text"

					# Find the first Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the first Positional_Accuracy, or make one

					set q [$m find_in $p Positional_Accuracy]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Positional_Accuracy]
						if {!$p} {
							puts stdout "Error: failed to insert Positional_Accuracy"
							exit
							}
						}

					# Inside that, find the first Horizontal_Positional_Accuracy, or make one

					set q [$m find_in $p Horizontal_Positional_Accuracy]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Horizontal_Positional_Accuracy]
						if {!$p} {
							puts stdout "Error: failed to insert Horizontal_Positional_Accuracy"
							exit
							}
						}

					# Inside that, find the first Horizontal_Positional_Accuracy_Report, or make one

					set q [$m find_in $p Horizontal_Positional_Accuracy_Report]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Horizontal_Positional_Accuracy_Report]
						if {!$p} {
							puts stdout "Error: failed to insert Horizontal_Positional_Accuracy_Report"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$horizpar"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $horizpar $s] != -1} {
							append s "  $horizpar"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Vertical Positional Accuracy Report:
				#---------------------------------------------------------

				if {[string compare $t "Vertical Positional Accuracy Report:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set vertaccr "$text"

					# Find the first Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the first Positional_Accuracy, or make one

					set q [$m find_in $p Positional_Accuracy]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Positional_Accuracy]
						if {!$p} {
							puts stdout "Error: failed to insert Positional_Accuracy"
							exit
							}
						}

					# Inside that, find the first Vertical_Positional_Accuracy, or make one

					set q [$m find_in $p Vertical_Positional_Accuracy]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Vertical_Positional_Accuracy]
						if {!$p} {
							puts stdout "Error: failed to insert Vertical_Positional_Accuracy"
							exit
							}
						}

					# Inside that, find the first Vertical_Positional_Accuracy_Report, or make one

					set q [$m find_in $p Vertical_Positional_Accuracy_Report]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Vertical_Positional_Accuracy_Report]
						if {!$p} {
							puts stdout "Error: failed to insert Vertical_Positional_Accuracy_Report"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$vertaccr"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $vertaccr $s] != -1} {
							append s "  $vertaccr"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Direct Spatial Reference Method:
				#---------------------------------------------------------

				if {[string compare $t "Direct Spatial Reference Method:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set direct "$text"

					# Find the first Spatial_Data_Organization_Information, or make one

					set q [$m find_first Spatial_Data_Organization_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Spatial_Data_Organization_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Spatial_Data_Organization_Information"
							exit
							}
						}

					# Inside that, find the first Positional_Accuracy, or make one

					set q [$m find_in $p Direct_Spatial_Reference_Method]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Direct_Spatial_Reference_Method]
						if {!$p} {
							puts stdout "Error: failed to insert Direct_Spatial_Reference_Method"
							exit
							}
						}

					# If there was no value, use the current text as the value

					set s [$m value_of -nonewline $p]
					if {[string length $s] == 0} {
						$m value_set $p "$direct"
						} \
					else {
						# If the new value is not contained in the old value, append
						# the new value to what was already there

						if {[string first $direct $s] != -1} {
							append s "  $direct"
							$m value_set $p "$s"
							}
						}

					continue
					}

				#---------------------------------------------------------
				# Latitude Resolution:
				#---------------------------------------------------------

				if {[string compare $t "Latitude Resolution:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set latres "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geographic, or make one

					set element "Geographic"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Latitude_Resolution, or make one

					set element "Latitude_Resolution"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Longitude Resolution:
				#---------------------------------------------------------

				if {[string compare $t "Longitude Resolution:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set longres "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geographic, or make one

					set element "Geographic"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Longitude_Resolution, or make one

					set element "Longitude_Resolution"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Geographic Coordinate Units:
				#---------------------------------------------------------

				if {[string compare $t "Geographic Coordinate Units:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set geogunit "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geographic, or make one

					set element "Geographic"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geographic_Coordinate_Units, or make one

					set element "Geographic_Coordinate_Units"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Horizontal Datum Name:
				#---------------------------------------------------------

				if {[string compare $t "Horizontal Datum Name:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set horizdn "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geodetic_Model, or make one

					set element "Geodetic_Model"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Datum_Name, or make one

					set element "Horizontal_Datum_Name"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Ellipsoid Name:
				#---------------------------------------------------------

				if {[string compare $t "Ellipsoid Name:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set ellips "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geodetic_Model, or make one

					set element "Geodetic_Model"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Ellipsoid_Name, or make one

					set element "Ellipsoid_Name"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Semi-major Axis:
				#---------------------------------------------------------

				if {[string compare $t "Semi-major Axis:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set semiaxis "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geodetic_Model, or make one

					set element "Geodetic_Model"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Semi-major_Axis, or make one

					set element "Semi-major_Axis"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Denominator of Flattening Ratio:
				#---------------------------------------------------------

				if {[string compare $t "Denominator of Flattening Ratio:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set denflat "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Horizontal_Coordinate_System_Definition, or make one

					set element "Horizontal_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Geodetic_Model, or make one

					set element "Geodetic_Model"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Denominator_of_Flattening_Ratio, or make one

					set element "Denominator_of_Flattening_Ratio"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Altitude Datum Name:
				#---------------------------------------------------------

				if {[string compare $t "Altitude Datum Name:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set altdatum "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Vertical_Coordinate_System_Definition, or make one

					set element "Vertical_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_System_Definition, or make one

					set element "Altitude_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_Datum_Name, or make one

					set element "Altitude_Datum_Name"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Altitude Resolution (Primary):
				#---------------------------------------------------------

				if {[string compare $t "Altitude Resolution (Primary):"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set altres "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Vertical_Coordinate_System_Definition, or make one

					set element "Vertical_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_System_Definition, or make one

					set element "Altitude_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_Resolution, or make one

					set element "Altitude_Resolution"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Altitude Distance Units:
				#---------------------------------------------------------

				if {[string compare $t "Altitude Distance Units:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set altunits "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Vertical_Coordinate_System_Definition, or make one

					set element "Vertical_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_System_Definition, or make one

					set element "Altitude_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_Distance_Units, or make one

					set element "Altitude_Distance_Units"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Altitude Encoding Method:
				#---------------------------------------------------------

				if {[string compare $t "Altitude Encoding Method:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set altenc "$text"

					# Find the first Spatial_Reference_Information, or make one

					set element "Spatial_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Vertical_Coordinate_System_Definition, or make one

					set element "Vertical_Coordinate_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_System_Definition, or make one

					set element "Altitude_System_Definition"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Altitude_Encoding_Method, or make one

					set element "Altitude_Encoding_Method"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Entity and Attribute Overview:
				#---------------------------------------------------------

				if {[string compare $t "Entity and Attribute Overview:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}

					# Remember this value, and insert it into the metadata when
					# you see the corresponding Entity_and_Attribute_Detail_Citation

					set eaover "$text"

					continue
					}

				#---------------------------------------------------------
				# Entity and Attribute Detail Citation:
				#---------------------------------------------------------

				if {[string compare $t "Entity and Attribute Detail Citation:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set eadetcit "$text"

					# Store this only if you have already encountered an Entity_and_Attribute_Overview

					if {[string length $eaover] > 0} {
						# Find the first Entity_and_Attribute_Information, or make one

						set element "Entity_and_Attribute_Information"
						set q [$m find_first $element]
						if {$q} {
							set p $q
							} \
						else {
							set p [$m insert -child $root $element]
							if {!$p} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Inside that, create an Overview_Description

						set element "Overview_Description"
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}

						# Inside that, create an Entity_and_Attribute_Overview

						set element "Entity_and_Attribute_Overview"
						set q [$m insert -child $p $element]
						if {!$q} {
							puts stdout "Error: failed to insert $element"
							exit
							}

						# Store the current value of eaover into it

						$m value_set $q "$eaover"

						# Alongside that, create an Entity_and_Attribute_Detail_Citation

						set element "Entity_and_Attribute_Detail_Citation"
						set q [$m insert -child $p $element]
						if {!$q} {
							puts stdout "Error: failed to insert $element"
							exit
							}

						# Store the current value of eadetcit into it

						$m value_set $q "$eadetcit"

						# Clear the value of eaover so it doesn't get used twice by mistake

						set eaover ""
						} \
					else {
						puts stdout "Warning: Encountered Entity_and_Attribute_Detail_Citation without"
						puts stdout "         first finding an Entity_and_Attribute_Overview"
						puts stdout "         The detail citation will be ignored."
						}

					continue
					}

				#---------------------------------------------------------
				# Sources
				#---------------------------------------------------------

				if {[string compare $t "Source Citation Abbreviation:"] == 0} {
					set srccitea {}
					set origin   {}
					set title    {}
					set pubdate  {}
					set begdate  {}
					set enddate  {}
					set srccontr {}
					set srcscale {}
					set typesrc  {}
					set caldate  {}
					set srccurr  {}

					incr here
					while {$here < $line_count && $indent($here) > $k} {
						set kk $indent($here)
						set t [string trim $lines($here)]
						switch $t {
							"Originator:" {
								incr here
								while {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										lappend origin $t
										}
									incr here
									}
								}
							"Title:" {
								incr here
								while {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										if {[string length $title] > 0} {
											append title " "
											}
										append title [string trim $lines($here)]
										}
									incr here
									}
								}
							"Publication Date:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set pubdate [string trim $lines($here)]
										}
									incr here
									}
								}
							"Beginning Date:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set begdate [string trim $lines($here)]
										}
									incr here
									}
								}
							"Ending Date:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set enddate [string trim $lines($here)]
										}
									incr here
									}
								}
							"Source Contribution:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set srccontr [string trim $lines($here)]
										}
									incr here
									}
								}
							"Source Scale Denominator:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set srcscale [string trim $lines($here)]
										}
									incr here
									}
								}
							"Type of Source Media:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set typesrc [string trim $lines($here)]
										}
									incr here
									}
								}
							"Calendar Date:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set caldate [string trim $lines($here)]
										}
									incr here
									}
								}
							"Source Currentness Reference:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set srccurr [string trim $lines($here)]
										}
									incr here
									}
								}
							default {
								set srccitea $t
								incr here
								}
							}
						}

					# Find the Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the Lineage, or make one

					set q [$m find_in $p Lineage]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Lineage]
						if {!$p} {
							puts stdout "Error: failed to insert Lineage"
							exit
							}
						}

					# Inside that, make a new Source_Information

					set p [$m insert -child $p Source_Information]
					if {!$p} {
						puts stdout "Error: failed to insert Source_Information"
						exit
						}

					set q [$m insert -child $p Source_Citation]
					set q [$m insert -child $q Citation_Information]
					if {[llength $origin] > 0} {
						foreach item $origin {
							set r [$m insert -child $q Originator]
							$m value_set $r "$item"
							}
						}
					if {[string length $title] > 0} {
						set r [$m insert -child $q Title]
						$m value_set $r "$title"
						}
					if {[string length $pubdate] > 0} {
						set r [$m insert -child $q Publication_Date]
						$m value_set $r "$pubdate"
						}
					if {[string length $srcscale] > 0} {
						set q [$m insert -child $p Source_Scale_Denominator]
						$m value_set $q "$srcscale"
						}
					if {[string length $typesrc] > 0} {
						set q [$m insert -child $p Type_of_Source_Media]
						$m value_set $q "$typesrc"
						}
					set q [$m insert -child $p Source_Time_Period_of_Content]
					set r [$m insert -child $q Time_Period_Information]
					if {[string length $caldate] > 0} {
						set r [$m insert -child $r Single_Date/Time]
						set u [$m insert -child $r Calendar_Date]
						$m value_set $u "$caldate"
						} \
					else {
						set r [$m insert -child $r Range_of_Dates/Times]
						set u [$m insert -child $r Beginning_Date]
						$m value_set $u "$begdate"
						set u [$m insert -child $r Ending_Date]
						$m value_set $u "$enddate"
						}
					if {[string length $srccurr] > 0} {
						set r [$m insert -child $q Source_Currentness_Reference]
						$m value_set $r "$srccurr"
						}
					if {[string length $srccitea] > 0} {
						set q [$m insert -child $p Source_Citation_Abbreviation]
						$m value_set $q "$srccitea"
						}
					if {[string length $srccontr] > 0} {
						set q [$m insert -child $p Source_Contribution]
						$m value_set $q "$srccontr"
						}
					continue
					}

				#---------------------------------------------------------
				# Process Steps
				#---------------------------------------------------------

				if {[string compare $t "Process Description:"] == 0} {
					set procdesc {}
					set procdate {}
					set srcused  {}
					set srcprod  {}
					set acomment {}

					set cntorg   {}
					set addrtype {}
					set address  {}
					set city     {}
					set state    {}
					set postal   {}
					set cntvoice {}
					set cntinst  {}

					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						set kk $indent($here)
						set t [string trim $lines($here)]
						switch $t {
							"Process Date:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set procdate [string trim $lines($here)]
										}
									incr here
									}
								}
							"Source Used Citation Abbreviation:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set srcused [string trim $lines($here)]
										}
									incr here
									}
								}
							"Source Produced Citation Abbreviation:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										set srcprod [string trim $lines($here)]
										}
									incr here
									}
								}
							"Analysts Comments:" {
								incr here
								if {$here < $line_count && $indent($here) > $kk} {
									set t [string trim $lines($here)]
									if {[string compare $t "Not Applicable"] != 0} {
										lappend acomment [string trim $lines($here)]
										}
									incr here
									}
								}
							"Contact Organization:" {
								set k $indent($here)
								incr here
								set cntorg [string trim $lines($here)]

								# k is the indentation of the label "Contact Organization:"
								# begin by looking at the next line, which probably contains its value
								incr here
								while {$here < $line_count && $indent($here) > $k} {
									set kk $indent($here)
									set t [string trim $lines($here)]
									switch $t {
										"Address Type:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set addrtype "$text"
											}
										"Address:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set address "$text"
											}
										"City:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set city "$text"
											}
										"State or Province:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set state "$text"
											}
										"Postal Code:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set postal "$text"
											}
										"Contact Voice Telephone:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set cntvoice "$text"
											}
										"Contact Instructions:" {
											set text ""
											incr here
											while {$here < $line_count && $indent($here) > $kk} {
												if {[string length $text] > 0} {
													append text " "
													}
												append text [string trim $lines($here)]
												incr here
												}
											set cntinst "$text"
											}
										default {
											append cntorg $t
											incr here
											}
										}
									}
								}
							default {
								if {[string length $procdesc] > 0} {
									append procdesc " "
									}
								append procdesc $t
								incr here
								}
							}
						}

					# Find the Data_Quality_Information, or make one

					set q [$m find_first Data_Quality_Information]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root Data_Quality_Information]
						if {!$p} {
							puts stdout "Error: failed to insert Data_Quality_Information"
							exit
							}
						}

					# Inside that, find the Lineage, or make one

					set q [$m find_in $p Lineage]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p Lineage]
						if {!$p} {
							puts stdout "Error: failed to insert Lineage"
							exit
							}
						}

					# Inside that, make a new Process_Step

					set p [$m insert -child $p Process_Step]
					if {!$p} {
						puts stdout "Error: failed to insert Process_Step"
						exit
						}

					if {[string length $procdesc] > 0} {
						set q [$m insert -child $p Process_Description]
						$m value_set $q "$procdesc"
						}
					if {[string length $srcused] > 0} {
						set q [$m insert -child $p Source_Used_Citation_Abbreviation]
						$m value_set $q "$srcused"
						}
					if {[string length $procdate] > 0} {
						set q [$m insert -child $p Process_Date]
						$m value_set $q "$procdate"
						}
					if {[string length $srcprod] > 0} {
						set q [$m insert -child $p Source_Produced_Citation_Abbreviation]
						$m value_set $q "$srcprod"
						}
					if {[llength $acomment] > 0} {
						foreach item $acomment {
							set q [$m insert -child $p Analysts_Comments]
							$m value_set $q "$item"
							}
						}

					# If contact information was given here, make a Process_Contact to hold it

					if {[llength $cntorg] > 0} {
						set q [$m insert -child $p Process_Contact]
						set p $q

						# Inside that, find the first Contact_Information, or make one

						set element "Contact_Information"
						set q [$m find_in $p $element]
						if {$q} {
							set p $q
							} \
						else {
							set p [$m insert -child $p $element]
							if {!$p} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# If the Contact_Organization was specified, store it

						if {[string length $cntorg] > 0} {
							# Inside that, find the first Contact_Organization_Primary, or make one

							set element "Contact_Organization_Primary"
							set q [$m find_in $p $element]
							if {!$q} {
								set q [$m insert -child $p $element]
								if {!$q} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Inside that, find the first Contact_Organization, or make one

							set element "Contact_Organization"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use cntorg as the new value

							$m value_set $r "$cntorg"
							}

						# If the Contact_Voice_Telephone was specified, store it

						if {[string length $cntvoice] > 0} {
							# Inside that, find the first Contact_Voice_Telephone, or make one

							set element "Contact_Voice_Telephone"
							set q [$m find_in $p $element]
							if {!$q} {
								set q [$m insert -child $p $element]
								if {!$q} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use cntvoice as the new value

							$m value_set $q "$cntvoice"
							}

						# If the Contact_Instructions was specified, store it

						if {[string length $cntinst] > 0} {
							# Inside that, find the first Contact_Instructions, or make one

							set element "Contact_Instructions"
							set q [$m find_in $p $element]
							if {!$q} {
								set q [$m insert -child $p $element]
								if {!$q} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use cntinst as the new value

							$m value_set $q "$cntinst"
							}

						# Find or make a Contact_Address

						set element "Contact_Address"
						set q [$m find_in $p $element]
						if {!$q} {
							set q [$m insert -child $p $element]
							if {!$q} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						if {[string length $addrtype] > 0} {
							# Inside that, find the first Address_Type, or make one

							set element "Address_Type"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use addrtype as the new value

							$m value_set $r $addrtype
							}

						if {[string length $address] > 0} {
							# Inside that, find the first Address, or make one

							set element "Address"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use address as the new value

							$m value_set $r "$address"
							}

						if {[string length $city] > 0} {
							# Inside that, find the first City, or make one

							set element "City"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use city as the new value

							$m value_set $r "$city"
							}

						if {[string length $state] > 0} {
							# Inside that, find the first State_or_Province, or make one

							set element "State_or_Province"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use state as the new value

							$m value_set $r "$state"
							}

						if {[string length $postal] > 0} {
							# Inside that, find the first Postal_Code, or make one

							set element "Postal_Code"
							set r [$m find_in $q $element]
							if {!$r} {
								set r [$m insert -child $q $element]
								if {!$r} {
									puts stdout "Error: failed to insert $element"
									exit
									}
								}

							# Use postal as the new value

							$m value_set $r "$postal"
							}
						}
					continue
					}

				#---------------------------------------------------------
				# Metadata Date and Metadata_Contact
				#---------------------------------------------------------

				if {[string compare $t "Metadata Date:"] == 0} {
					set cntorg   {}
					set addrtype {}
					set address  {}
					set city     {}
					set state    {}
					set postal   {}
					set cntvoice {}
					set cntinst  {}

					# First value, indented, is the metadata date
					incr here
					set metd [string trim $lines($here)]

					# Next value is the Contact_Organization
					incr here
					set t [string trim $lines($here)]
					set k $indent($here)

					if {[string compare $t "Contact Organization:"] == 0} {
						# k is the indentation of the label "Contact Organization:"
						# begin by looking at the next line, which probably contains its value
						incr here
						while {$here < $line_count && $indent($here) > $k} {
							set kk $indent($here)
							set t [string trim $lines($here)]
							switch $t {
								"Address Type:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set addrtype "$text"
									}
								"Address:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set address "$text"
									}
								"City:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set city "$text"
									}
								"State or Province:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set state "$text"
									}
								"Postal Code:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set postal "$text"
									}
								"Contact Voice Telephone:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set cntvoice "$text"
									}
								"Contact Instructions:" {
									set text ""
									incr here
									while {$here < $line_count && $indent($here) > $kk} {
										if {[string length $text] > 0} {
											append text " "
											}
										append text [string trim $lines($here)]
										incr here
										}
									set cntinst "$text"
									}
								default {
									set cntorg $t
									incr here
									}
								}
							}
						}

					# Find the first Metadata_Reference_Information, or make one

					set element "Metadata_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Metadata_Date, or make one

					set element "Metadata_Date"
					set q [$m find_in $p $element]
					if {!$q} {
						set q [$m insert -child $p $element]
						if {!$q} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $q "$metd"

					# Alongside that, find the first Metadata_Contact, or make one

					set element "Metadata_Contact"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Contact_Information, or make one

					set element "Contact_Information"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# If the Contact_Organization was specified, store it

					if {[string length $cntorg] > 0} {
						# Inside that, find the first Contact_Organization_Primary, or make one

						set element "Contact_Organization_Primary"
						set q [$m find_in $p $element]
						if {!$q} {
							set q [$m insert -child $p $element]
							if {!$q} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Inside that, find the first Contact_Organization, or make one

						set element "Contact_Organization"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use cntorg as the new value

						$m value_set $r "$cntorg"
						}

					# If the Contact_Voice_Telephone was specified, store it

					if {[string length $cntvoice] > 0} {
						# Inside that, find the first Contact_Voice_Telephone, or make one

						set element "Contact_Voice_Telephone"
						set q [$m find_in $p $element]
						if {!$q} {
							set q [$m insert -child $p $element]
							if {!$q} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use cntvoice as the new value

						$m value_set $q "$cntvoice"
						}

					# If the Contact_Instructions was specified, store it

					if {[string length $cntinst] > 0} {
						# Inside that, find the first Contact_Instructions, or make one

						set element "Contact_Instructions"
						set q [$m find_in $p $element]
						if {!$q} {
							set q [$m insert -child $p $element]
							if {!$q} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use cntinst as the new value

						$m value_set $q "$cntinst"
						}

					# Find or make a Contact_Address

					set element "Contact_Address"
					set q [$m find_in $p $element]
					if {!$q} {
						set q [$m insert -child $p $element]
						if {!$q} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					if {[string length $addrtype] > 0} {
						# Inside that, find the first Address_Type, or make one

						set element "Address_Type"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use addrtype as the new value

						$m value_set $r $addrtype
						}

					if {[string length $address] > 0} {
						# Inside that, find the first Address, or make one

						set element "Address"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use address as the new value

						$m value_set $r "$address"
						}

					if {[string length $city] > 0} {
						# Inside that, find the first City, or make one

						set element "City"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use city as the new value

						$m value_set $r "$city"
						}

					if {[string length $state] > 0} {
						# Inside that, find the first State_or_Province, or make one

						set element "State_or_Province"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use state as the new value

						$m value_set $r "$state"
						}

					if {[string length $postal] > 0} {
						# Inside that, find the first Postal_Code, or make one

						set element "Postal_Code"
						set r [$m find_in $q $element]
						if {!$r} {
							set r [$m insert -child $q $element]
							if {!$r} {
								puts stdout "Error: failed to insert $element"
								exit
								}
							}

						# Use postal as the new value

						$m value_set $r "$postal"
						}

					continue
					}

				#---------------------------------------------------------
				# Metadata Standard Name
				#---------------------------------------------------------

				if {[string compare $t "Metadata Standard Name:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set metstdn "$text"

					# Find the first Metadata_Reference_Information, or make one

					set element "Metadata_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Metadata_Standard_Name, or make one

					set element "Metadata_Standard_Name"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				#---------------------------------------------------------
				# Metadata Standard Version
				#---------------------------------------------------------

				if {[string compare $t "Metadata Standard Version:"] == 0} {
					set text ""
					set k $indent($here)
					incr here
					while {$here < $line_count && $indent($here) > $k} {
						if {[string length $text] > 0} {
							append text " "
							}
						append text [string trim $lines($here)]
						incr here
						}
					set metstdv "$text"

					# Find the first Metadata_Reference_Information, or make one

					set element "Metadata_Reference_Information"
					set q [$m find_first $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $root $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Inside that, find the first Metadata_Standard_Version, or make one

					set element "Metadata_Standard_Version"
					set q [$m find_in $p $element]
					if {$q} {
						set p $q
						} \
					else {
						set p [$m insert -child $p $element]
						if {!$p} {
							puts stdout "Error: failed to insert $element"
							exit
							}
						}

					# Use the new text as the value

					$m value_set $p "$text"

					continue
					}

				incr here
				}

			# Insert incomplete Entity_and_Attribute_Overview, if any

			if {[string length $eaover] > 0} {
				# Find the first Entity_and_Attribute_Information, or make one

				set element "Entity_and_Attribute_Information"
				set q [$m find_first $element]
				if {$q} {
					set p $q
					} \
				else {
					set p [$m insert -child $root $element]
					if {!$p} {
						puts stdout "Error: failed to insert $element"
						exit
						}
					}

				# Inside that, create an Overview_Description

				set element "Overview_Description"
				set p [$m insert -child $p $element]
				if {!$p} {
					puts stdout "Error: failed to insert $element"
					exit
					}

				# Inside that, create an Entity_and_Attribute_Overview

				set element "Entity_and_Attribute_Overview"
				set q [$m insert -child $p $element]
				if {!$q} {
					puts stdout "Error: failed to insert $element"
					exit
					}

				# Store the current value of eaover into it

				$m value_set $q "$eaover"

				# Warn user that this should not have happened

				puts "Warning: an Entity_and_Attribute_Overview was found but there is"
				puts "         no corresponding Entity_and_Attribute_Detail_Citation"
				}

			# Write out the metadata
			$m write "merged-$filename"
			}
		$m forget
		} \
	else {
		puts stdout "Error: could not open boilerplate metadata file $boilerplate"
		exit
		}
	}
