# Script to create a batch file of commands for processing
# all of the NHD metadata in the current directory

set out [open "doit.bat" w]

# Try to figure out what HUC we're actually in
set path [file split [pwd]]
set n [llength $path]
set this_huc [lindex $path [expr $n - 2]]


# Find all of the .met files
set met_list [glob *.met]

# Find all of the 8-digit HUCs
set huc {}
foreach f $met_list {
	if {[regexp "\^\[0-9\]\{8\}001.met\$" $f]} {
		lappend huc [string range $f 0 7]
		}
	}

# For each HUC, find all of the .met files pertaining to it
foreach h $huc {
	set record {}
	foreach f $met_list {
		if {[string first $h $f] != -1} {
			lappend record $f
			}
		}
	if {$h == $this_huc} {
		if {[llength $record] > 0} {
			# Start by copying the NHD boilerplate to nhd0.met
			puts "copy nhd-hr.met nhd0.met"
			puts $out "copy nhd-hr.met nhd0.met"
			# Merge each record with nhd0.met,
			# then rename the result to nhd0.met
			foreach r $record {
				puts "tclsh merge.tcl $r"
				puts $out "tclsh merge.tcl $r"
				puts "move merged-$r nhd0.met"
				puts $out "move merged-$r nhd0.met"
				}
			# Now the result of all the merges is in nhd0.met;
			# rename this to <huc>.met
			puts "move nhd0.met $h.met"
			puts $out "move nhd0.met $h.met"
			}
		}
	}
close $out

# end
